/*
 *
 * protocol.c
 *	モータとの通信を扱う
 *
 *
 * Copyright 2008-2009 
 * Uchibori Laboratory, Numazu National College of Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#include <assert.h>
#include <errno.h>
#include <unistd.h>
#include <mg3/motor.h>


int motor_send_control_pkt (int fd, uint8_t speed, uint8_t kp, uint8_t ki)
{
	struct {
		uint8_t command;
		uint8_t spped;
		uint8_t kp;
		uint8_t ki;
	} packet = {MG3_MOTOR_PKT_SETCTL, speed, kp, ki};
	int err;
	uint8_t response; 

	// パケット長が2であるか確認する.
	assert(sizeof(packet) == 2);

	// パケットを送信する.
	err = write(fd, &packet, sizeof(packet));
	if ( err < 0 ) {
		return err;
	}

	// 返答を受信する．
	err = read(fd, &response, sizeof(response));
	if ( err < 0 ) {
		return err;
	}

	// 返答がACKであるか確認する.
	if ( response != (MG3_MOTOR_PKT_SETCTL | MG3_MOTOR_PKT_ACK) ) {
		errno = EBADRQC; // 不正なリクエストコード
		return -1;
	}

	return 0;
}












