/*
 *
 * fsync.c
 *	test fsync()
 *
 *
 * Copyright 2008-2009 
 * Uchibori Laboratory, Numazu National College of Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <motor/sci_ctl.h>
#include "../sci_config.h"
#include "devfile.h"


int fd;


static void *rd_func (void *arg)
{
	char c;

	arg = NULL; // fake gcc

	while ( 1 ) {
		read(fd, &c, 1);
	}

	return 0;
}



#define BUFLEN (MG3_SCI_FIFO_SIZE * 2)

int main (void)
{
	pthread_t tid;
	int err, i, size;
	char buf[BUFLEN];

	fprintf(stderr, "testing fsync");
	fd = open(FNAME, O_RDWR);
	if ( fd < 0 ) {
		perror("Open");
		return -1;
	}

	err = pthread_create(&tid, NULL, rd_func, NULL);
	if ( err ) {
		errno = err;
		perror("Create Thread");
		return -1;
	}


	for ( i = 0; i < BUFLEN; i++ ) {
		buf[i] = i;
	}

	err = write(fd, buf, BUFLEN);
	assert(err == BUFLEN);

	err = ioctl(fd, TIOCOUTQ, &size);
	assert(err == 0);
	fprintf(stderr, "(%d)...", size);

	// sync
	err = fsync(fd);
	assert(err == 0);
	err = ioctl(fd, TIOCOUTQ, &size);
	assert(err == 0);
	assert(size == 0);
	


	fprintf(stderr, "ok.\n");

	return 0;
}




