/*
 *
 * rw.c
 *	test read(), write()
 *
 *
 * Copyright 2008-2009 
 * Uchibori Laboratory, Numazu National College of Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <motor/sci_ctl.h>
#include "../sci_config.h"
#include "devfile.h"


#define TESTC 0x23

static int rw_1 (int fd)
{
	unsigned char c;


	fprintf(stderr, "  writing from invalid buffer...");
	write(fd, NULL, 10);
	if ( errno == EFAULT ) {
		fprintf(stderr, "ok.\n");
	}
	else {
		fprintf(stderr, "failed.\n");
		return -1;
	}


	fprintf(stderr, "  writing from valid buffer...");
	errno = 0;
	c = TESTC;
	if ( write(fd, &c, 1) == 1 ) {
		fprintf(stderr, "ok.\n");
	}
	else {
		perror("write");
		return -1;
	}

	fprintf(stderr, "  reading to invalid buffer...");
	errno = 0;
	read(fd, NULL, 1);
	if ( errno == EFAULT ) {
		fprintf(stderr, "ok.\n");
	}
	else {
		fprintf(stderr, "failed.\n");
		return -1;
	}
	
	fprintf(stderr, "  reading to valid buffer...");
	if ( read(fd, &c, 1) != 1 ) {
		perror("read");
		return -1;
	}
	if ( c == TESTC ) {
		fprintf(stderr, "ok.\n");
	}
	else {
		fprintf(stderr, "failed(%02x).\n", (unsigned)c);
	}

	return 0;
}




static int rw_4 (int fd)
{
	uint32_t send_data = 0x04030201;
	uint32_t recv_data;
	size_t data_size = sizeof(uint32_t);

	fprintf(stderr, "  writing %u byte...", data_size);
	errno = 0;
	if ( write(fd, &send_data, data_size) == (ssize_t)data_size ) {
		fprintf(stderr, "ok.\n");
	}
	else {
		perror("write");
		return -1;
	}

#if 1
	fprintf(stderr, "  reading %u byte...", data_size);
	if ( read(fd, &recv_data, data_size) != (ssize_t)data_size ) {
		perror("read");
		return -1;
	}
	if ( send_data == recv_data ) {
		fprintf(stderr, "ok.\n");
	}
	else {
		fprintf(stderr, "failed.\n");
	}
#else
	sleep(5);
#endif

	return 0;
}



#define BUF_SIZE (MG3_SCI_FIFO_SIZE * 3)
static int rw_n (int fd)
{
	char wbuf[BUF_SIZE], rbuf[BUF_SIZE];
	int i, result;


	for ( i = 0; i < BUF_SIZE; ++i ) {
		wbuf[i] = i;
	}

	fprintf(stderr, "  writing %u byte...", BUF_SIZE);
	errno = 0;
	result = write(fd, wbuf, BUF_SIZE);
	if ( result == BUF_SIZE ) {
		fprintf(stderr, "?.\n");
	}
	else if ( result > 0 && result < BUF_SIZE ) {
		result = write(fd, wbuf, BUF_SIZE);
		if ( result < 0 && errno == EIO ) {
			fprintf(stderr, "ok.\n");
		}
		else {
			perror("write");
			return -1;
		}
	}
	else {
		perror("write");
		return -1;
	}

#if 1
	// 書き込みがMG3_SCI_FIFO_SIZE[Byte]を越えているので，
	// MG3_SCI_FIFO_SIZE[Byte]以上読み込もうとすると，
	// EIOエラーが返るはず．
	fprintf(stderr, "  reading %u byte...", BUF_SIZE);
	result = read(fd, rbuf, BUF_SIZE);
	if ( result != MG3_SCI_FIFO_SIZE ) {
		perror("read");
		fprintf(stderr, "read %d byte", result);
		return -1;
	}
	if ( memcmp(rbuf, wbuf, MG3_SCI_FIFO_SIZE) != 0 ) {
		fprintf(stderr, "failed.\n");
		return -1;
	}
	if ( read(fd, rbuf, BUF_SIZE) != -1 ) {
		fprintf(stderr, "failed.\n");
		return -1;
	}
	if ( errno == EIO ) {
		fprintf(stderr, "ok.\n");
	}
#else
	sleep(5);
#endif

	return 0;
}



int main (void)
{
	int fd;

	fprintf(stderr, "testing read()/write()...\n");

	fd = open(FNAME, O_RDWR);
	if ( fd < 0 ) {
		perror("open");
		return -1;
	}

//	ioctl(fd, MG3_SCI_IOENDMSG);

	if ( rw_1(fd) != 0 ) {
		return -1;
	}

	if ( rw_4(fd) != 0 ) {
		return -2;
	}

	if ( rw_n(fd) != 0 ) {
		return -3;
	}

	close(fd);
	
	fprintf(stderr, "done.\n");

	return 0;
}

