/*
 *
 * select.c
 *	test select()
 *
 *
 * Copyright 2008-2009 
 * Uchibori Laboratory, Numazu National College of Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "devfile.h"





int main (void)
{
	int fd, result;
	const unsigned long wtime = 100000; // 100[ms]
	double wsec;              // 待った時間[s]
	struct timeval wait_time; // 最大待ち時間
	fd_set rfds;              // どのシリアルポートも待つのか指定するためのもの
	char wc, rc;


	fprintf(stderr, "testing select()...\n");

	fd = open(FNAME, O_RDWR);
	if ( fd < 0 ) {
		perror("open");
		return -1;
	}

	// 何も書かずにread()待ち．
	fprintf(stderr, "  waiting for ready to read with no written data...");
	wait_time.tv_sec = 0;
	wait_time.tv_usec = wtime;
	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);

	result = select(fd + 1, &rfds, NULL, NULL, &wait_time);
	if ( result != 0 ) {
		fprintf(stderr, "failed.\n");
	}
	else {
		assert(wait_time.tv_usec == 0);
		wsec = (wtime - wait_time.tv_usec) / 1000000.0;
		fprintf(stderr, "waited %.3f[sec] ok.\n", wsec);
	}


	// 1文字書いてすぐにread()待ち．
	fprintf(stderr, "  waiting for ready to read with written data...");
	wait_time.tv_sec = 0;
	wait_time.tv_usec = wtime;
	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);

	wc = 0xa5;
	write(fd, &wc, 1);
	result = select(fd + 1, &rfds, NULL, NULL, &wait_time);
	if ( result == 0 ) {
		fprintf(stderr, "failed.\n");
	}
	else if ( !FD_ISSET(fd, &rfds) ) {
		fprintf(stderr, "failed.\n");
	}
	else {
		assert(wait_time.tv_usec > 0);
		wsec = (wtime - wait_time.tv_usec) / 1000000.0;
		fprintf(stderr, "waited %.3f[sec] ok.\n", wsec);
	}
	rc = 0;
	read(fd, &rc, 1);
	assert(rc == wc);


	// 1文字書いたデータが到着してからread()待ち．
	fprintf(stderr, "  waiting for ready to read with written data...");
	wait_time.tv_sec = 0;
	wait_time.tv_usec = wtime;
	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);

	wc = 0x5a;
	write(fd, &wc, 1);
	usleep(10000);
	result = select(fd + 1, &rfds, NULL, NULL, &wait_time);
	if ( result == 0 ) {
		fprintf(stderr, "failed.\n");
	}
	else if ( !FD_ISSET(fd, &rfds) ) {
		fprintf(stderr, "failed.\n");
	}
	else {
		assert(wait_time.tv_usec <= (long)wtime);
		wsec = (wtime - wait_time.tv_usec) / 1000000.0;
		fprintf(stderr, "waited %.3f[sec] ok.\n", wsec);
	}
	rc = 0;
	read(fd, &rc, 1);
	assert(rc == wc);


	close(fd);

	fprintf(stderr, "done.\n");

	
	return 0;
}

