/*
 *
 * motor.h
 *	Control Motor
 *
 *
 * Copyright 2008-2009 
 * Uchibori Laboratory, Numazu National College of Technology
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#ifndef MG3_MOTOR_H
#define MG3_MOTOR_H

#include <sys/types.h>
#include <stdint.h>

//
// Low level handler
//

/*
 * Protocol
 * # Reset
 * Req: 0x00
 * Res: 0x80
 *
 * # 制御パラメータの設定
 * Req: 0x01, 目標値, Kp, Ki
 * Res: 0x81
 *  目標値: -128〜127(パルス/100ms)
 *  Kp: 0〜255
 *  Ki: 0〜255
 *
 * # モータの状態取得
 * Req: 0x02
 * Res: 0x82, 速度, PWMデューティ(2Byte), 累積カウント(4Byte)
 *  速度: -128〜127[パルス/100ms]
 *  PWM duty: 0〜255 (duty比: 0〜100%)(要検討)
 *  累積カウント: -2^31〜2^31-1[パルス]
 *
 * # スタート
 * Req: 0x04
 * Res: 0x84
 *
 * # ストップ
 * Req: 0x08
 * Res: 0x88
 *
 * # 制御周期の変更
 * Req: 0x20, c0, c1
 * 周期 = (2^16 - ((c1 << 8) | c0)) * 1.6 [usec]
 * Res: 0xa0
 *
 * # Unknown
 * Req: 0x??
 * Res: 0x40 | Req
 */

#define MG3_MOTOR_PKT_PING  	0x00
#define MG3_MOTOR_PKT_SETCTL	0x01
#define MG3_MOTOR_PKT_GETST 	0x02
#define MG3_MOTOR_PKT_START 	0x04
#define MG3_MOTOR_PKT_STOP  	0x08
//#define MG3_MOTOR_PKT_GETCTL	0x10
#define MG3_MOTOR_PKT_PERIOD  	0x20
#define MG3_MOTOR_PKT_ACK   	0x80




int motor_send_control_packet (int fd, uint8_t speed, uint8_t kp, uint8_t ki);
//int motor_recv_control_packet (int fd, uint8_t *speed, uint8_t *kp, uint8_t *ki);
int motor_recv_status_packet (int fd, int8_t *speed, uint16_t *pwm, int32_t *round);






#endif // MG3_MOTOR_H

